@@ -223,10 +223,10 @@ my $term  = $input;
 my @c = ();
 if($input=~/C[0-9]+/) {
     push @c, $input;
-    ($term) = $umls->getSabDefConcepts($input);
+    ($term) = $umls->getConceptList($input);
 }
 else {
-    @c = $umls->getSabDefConcepts($input);
+    @c = $umls->getConceptList($input);
 }
 
 my $printFlag = 0;
@@ -310,7 +310,7 @@ sub showHelp() {
 #  function to output the version number
 ##############################################################################
 sub showVersion {
-    print '$Id: getCuiDef.pl,v 1.11 2010/05/24 23:05:10 btmcinnes Exp $';
+    print '$Id: getCuiDef.pl,v 1.12 2010/06/08 12:39:01 btmcinnes Exp $';
     print "\nCopyright (c) 2008, Ted Pedersen & Bridget McInnes\n";
 }
 
